searchNodes=[{"ref":"Cloak.html","title":"Cloak","type":"module","doc":"Cloak consists of two behaviours: Cloak.Cipher - Ciphers encrypt and decrypt data using a particular encryption algorithm and key. Cloak.Vault - Vaults configure and use ciphers. Applications can have multiple vaults."},{"ref":"Cloak.InvalidConfig.html","title":"Cloak.InvalidConfig","type":"exception","doc":"Raised when there is invalid configuration."},{"ref":"Cloak.MissingCipher.html","title":"Cloak.MissingCipher","type":"exception","doc":"Indicates that no cipher was found to encrypt/decrypt a given text."},{"ref":"Cloak.Cipher.html","title":"Cloak.Cipher","type":"behaviour","doc":"A behaviour for encryption/decryption modules. You can rely on this behaviour to create your own Cloak-compatible cipher modules. Example We will create a cipher that simply prepends &quot;Hello, &quot; to any given plaintext on encryption, and removes the prefix on decryption. First, define your own cipher module, and specify the Cloak.Cipher behaviour. defmodule MyApp.PrefixCipher do @behaviour Cloak.Cipher end Add some configuration to your vault for this new cipher: config :my_app, MyApp.Vault, ciphers: [ prefix: {MyApp.PrefixCipher, prefix: &quot;Hello, &quot;} ] The keyword list containing the :prefix will be passed in as opts to our cipher callbacks. You should specify any options your cipher will need for encryption/decryption here, such as the key. Next, define the can_decrypt?/2 callback: @impl true def can_decrypt?(ciphertext, opts) do String.starts_with?(ciphertext, opts[:prefix]) end If the ciphertext starts with &quot;Hello, &quot;, we know it was encrypted with this cipher and we can proceed. Finally, define the encrypt and decrypt functions: @impl true def encrypt(plaintext, opts) do opts[:prefix] &lt;&gt; plaintext end @impl true def decrypt(ciphertext, opts) do String.replace(ciphertext, opts[:prefix], &quot;&quot;) end You can now use your cipher with your vault! MyApp.Vault.encrypt!(&quot;World!&quot;, :prefix) # =&gt; &quot;Hello, World!&quot; MyApp.Vault.decrypt!(&quot;Hello, World!&quot;) # =&gt; &quot;World!&quot;"},{"ref":"Cloak.Cipher.html#c:can_decrypt?/2","title":"Cloak.Cipher.can_decrypt?/2","type":"callback","doc":"Determines if a given ciphertext can be decrypted by this cipher. Options are derived from the cipher configuration. See encrypt/2."},{"ref":"Cloak.Cipher.html#c:decrypt/2","title":"Cloak.Cipher.decrypt/2","type":"callback","doc":"Decrypt a value, using the given opts. Options are derived from the cipher configuration. See encrypt/2."},{"ref":"Cloak.Cipher.html#c:encrypt/2","title":"Cloak.Cipher.encrypt/2","type":"callback","doc":"Encrypt a value, using the given keyword list of options. These options derive from the cipher configuration, like so: config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;1, 0, ...&gt;&gt;} ] The above configuration will result in the following opts being passed to this function: [tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;1, 0, ...&gt;&gt;] Your implementation must include any information it will need for decryption in the generated ciphertext."},{"ref":"Cloak.Cipher.html#t:ciphertext/0","title":"Cloak.Cipher.ciphertext/0","type":"type","doc":""},{"ref":"Cloak.Cipher.html#t:opts/0","title":"Cloak.Cipher.opts/0","type":"type","doc":""},{"ref":"Cloak.Cipher.html#t:plaintext/0","title":"Cloak.Cipher.plaintext/0","type":"type","doc":""},{"ref":"Cloak.Vault.html","title":"Cloak.Vault","type":"behaviour","doc":"Encrypts and decrypts data, using a configured cipher. Create Your Vault Define a module in your application that uses Cloak.Vault. defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app end Configuration The :otp_app option should point to an OTP application that has the vault configuration. For example, the vault: defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app end Could be configured with Mix configuration like so: config :my_app, MyApp.Vault, json_library: Jason, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;...&gt;&gt;} ] The configuration options are: :json_library: Used to convert data types like lists and maps into binary so that they can be encrypted. (Default: Jason) :ciphers: a list of Cloak.Cipher modules the following format: {:label, {CipherModule, opts}} The first configured cipher in the list is the default for encrypting all new data, regardless of its label. This behaviour can be overridden on a field-by-field basis. The opts are specific to each cipher module. Check their codumentation for what each cipher requires. Cloak.Ciphers.AES.GCM Cloak.Ciphers.AES.CTR Runtime Configuration Because Vaults are GenServers, they can be configured at runtime using the init/1 callback. This allows you to easily fetch values like environment variables in a reliable way. The configuration from the :otp_app is passed as the first argument to the callback, allowing you to append to or change it at will. defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl GenServer def init(config) do config = Keyword.put(config, :ciphers, [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: decode_env!(&quot;CLOAK_KEY&quot;)} ]) {:ok, config} end defp decode_env!(var) do var |&gt; System.get_env() |&gt; Base.decode64!() end end You can also pass configuration to vaults via start_link/1: MyApp.Vault.start_link(ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: key} ]) Supervision Because Vaults are GenServers, you&#39;ll need to add your vault to your supervision tree in application.ex or whichever supervisor you prefer. children = [ MyApp.Vault ] If you want to pass in configuration values at runtime, you can do so: children = [ {MyApp.Vault, ciphers: [...]} ] Usage You can use the vault directly by calling its functions. MyApp.Vault.encrypt(&quot;plaintext&quot;) # =&gt; {:ok, &lt;&lt;...&gt;&gt;} MyApp.Vault.decrypt(ciphertext) # =&gt; {:ok, &quot;plaintext&quot;} See the documented callbacks below for the functions you can call. Performance Notes Vaults are not bottlenecks. They simply store configuration in an ETS table named after the Vault, e.g. MyApp.Vault.Config. All encryption and decryption is performed in your local process, reading configuration from the vault&#39;s ETS table."},{"ref":"Cloak.Vault.html#c:decrypt/1","title":"Cloak.Vault.decrypt/1","type":"callback","doc":"Decrypts a binary with the configured cipher that generated the binary. Automatically detects which cipher to use, based on the ciphertext."},{"ref":"Cloak.Vault.html#c:decrypt!/1","title":"Cloak.Vault.decrypt!/1","type":"callback","doc":"Like decrypt/1, but raises any errors."},{"ref":"Cloak.Vault.html#c:encrypt/1","title":"Cloak.Vault.encrypt/1","type":"callback","doc":"Encrypts a binary using the first configured cipher in the vault&#39;s configured :ciphers list."},{"ref":"Cloak.Vault.html#c:encrypt/2","title":"Cloak.Vault.encrypt/2","type":"callback","doc":"Encrypts a binary using the vault&#39;s configured cipher with the corresponding label."},{"ref":"Cloak.Vault.html#c:encrypt!/1","title":"Cloak.Vault.encrypt!/1","type":"callback","doc":"Like encrypt/1, but raises any errors."},{"ref":"Cloak.Vault.html#c:encrypt!/2","title":"Cloak.Vault.encrypt!/2","type":"callback","doc":"Like encrypt/2, but raises any errors."},{"ref":"Cloak.Vault.html#c:json_library/0","title":"Cloak.Vault.json_library/0","type":"callback","doc":"The JSON library the vault uses to convert maps and lists into JSON binaries before encryption."},{"ref":"Cloak.Vault.html#t:ciphertext/0","title":"Cloak.Vault.ciphertext/0","type":"type","doc":""},{"ref":"Cloak.Vault.html#t:label/0","title":"Cloak.Vault.label/0","type":"type","doc":""},{"ref":"Cloak.Vault.html#t:plaintext/0","title":"Cloak.Vault.plaintext/0","type":"type","doc":""},{"ref":"Cloak.Ciphers.AES.CTR.html","title":"Cloak.Ciphers.AES.CTR","type":"module","doc":"A Cloak.Cipher which encrypts values with the AES cipher in CTR (stream) mode. Internally relies on Erlang&#39;s :crypto.stream_encrypt/2."},{"ref":"Cloak.Ciphers.AES.CTR.html#can_decrypt?/2","title":"Cloak.Ciphers.AES.CTR.can_decrypt?/2","type":"function","doc":"Callback implementation for Cloak.Cipher.can_decrypt?2. Determines if a ciphertext can be decrypted with this cipher."},{"ref":"Cloak.Ciphers.AES.CTR.html#decrypt/2","title":"Cloak.Ciphers.AES.CTR.decrypt/2","type":"function","doc":"Callback implementation for Cloak.Cipher.decrypt/2. Decrypts a value encrypted with AES in CTR mode. Uses the key tag to find the correct key for decryption, and the IV included in the header to decrypt the body of the ciphertext. Parameters ciphertext - Binary ciphertext generated by encrypt/2. Examples iex&gt; encrypt(&quot;Hello&quot;) |&gt; decrypt &quot;Hello&quot;"},{"ref":"Cloak.Ciphers.AES.CTR.html#encrypt/2","title":"Cloak.Ciphers.AES.CTR.encrypt/2","type":"function","doc":"Callback implementation for Cloak.Cipher.encrypt. Encrypts a value using AES in CTR mode. Generates a random IV for every encryption, and prepends the key tag and IV to the beginning of the ciphertext. The format can be diagrammed like this: +-----------------------------------+----------------------+ | HEADER | BODY | +-------------------+---------------+----------------------+ | Key Tag (n bytes) | IV (16 bytes) | Ciphertext (n bytes) | +-------------------+---------------+----------------------+ | |__________________________________ | | +---------------+-----------------+-------------------+ | Type (1 byte) | Length (1 byte) | Key Tag (n bytes) | +---------------+-----------------+-------------------+ The Key Tag component of the header breaks down into a Type, Length, and Value triplet for easy decoding."},{"ref":"Cloak.Ciphers.AES.GCM.html","title":"Cloak.Ciphers.AES.GCM","type":"module","doc":"A Cloak.Cipher which encrypts values with the AES cipher in GCM (block) mode. Internally relies on Erlang&#39;s :crypto.block_encrypt/4."},{"ref":"Cloak.Ciphers.AES.GCM.html#can_decrypt?/2","title":"Cloak.Ciphers.AES.GCM.can_decrypt?/2","type":"function","doc":"Callback implementation for Cloak.Cipher.can_decrypt?/2. Determines whether this module can decrypt the given ciphertext."},{"ref":"Cloak.Ciphers.AES.GCM.html#decrypt/2","title":"Cloak.Ciphers.AES.GCM.decrypt/2","type":"function","doc":"Callback implementation for Cloak.Cipher.decrypt/2. Decrypts a value encrypted with AES in GCM mode."},{"ref":"Cloak.Ciphers.AES.GCM.html#encrypt/2","title":"Cloak.Ciphers.AES.GCM.encrypt/2","type":"function","doc":"Callback implementation for Cloak.Cipher.encrypt/2. Encrypts a value using AES in GCM mode. Generates a random IV for every encryption, and prepends the key tag, IV, and ciphertag to the beginning of the ciphertext. The format can be diagrammed like this: +----------------------------------------------------------+----------------------+ | HEADER | BODY | +-------------------+---------------+----------------------+----------------------+ | Key Tag (n bytes) | IV (16 bytes) | Ciphertag (16 bytes) | Ciphertext (n bytes) | +-------------------+---------------+----------------------+----------------------+ | |_________________________________ | | +---------------+-----------------+-------------------+ | Type (1 byte) | Length (1 byte) | Key Tag (n bytes) | +---------------+-----------------+-------------------+ The Key Tag component of the header breaks down into a Type, Length, and Value triplet for easy decoding. Important: Because a random IV is used for every encryption, encrypt/2 will not produce the same ciphertext twice for the same value."},{"ref":"Cloak.Ciphers.Deprecated.AES.CTR.html","title":"Cloak.Ciphers.Deprecated.AES.CTR","type":"module","doc":"DEPRECATED version of the Cloak.Ciphers.AES.CTR cipher, for use in migrating existing data to the new format used by Cloak.Ciphers.AES.CTR. Rationale The old Cloak.AES.CTR cipher used the following format for ciphertext: +---------------------------------------------------------+----------------------+ | HEADER | BODY | +----------------------+------------------+---------------+----------------------+ | Module Tag (n bytes) | Key Tag (1 byte) | IV (16 bytes) | Ciphertext (n bytes) | +----------------------+------------------+---------------+----------------------+ The new Cloak.Ciphers.AES.CTR implementation no longer prepends the &quot;Module Tag&quot; component, and uses a new format as described in its docs. This cipher can assist in upgrading old ciphertext to the new format. See the Upgrading from 0.6.x guide for usage."},{"ref":"Cloak.Ciphers.Deprecated.AES.GCM.html","title":"Cloak.Ciphers.Deprecated.AES.GCM","type":"module","doc":"DEPRECATED version of the Cloak.Ciphers.AES.GCM cipher, for use in migrating existing data to the new format used by Cloak.Ciphers.AES.GCM. Rationale The old Cloak.AES.GCM cipher used the following format for ciphertext: +---------------------------------------------------------+----------------------+ | HEADER | BODY | +----------------------+------------------+---------------+----------------------+ | Module Tag (n bytes) | Key Tag (1 byte) | IV (16 bytes) | Ciphertext (n bytes) | +----------------------+------------------+---------------+----------------------+ The new Cloak.Ciphers.AES.GCM implementation no longer prepends the &quot;Module Tag&quot; component, and uses a new format as described in its docs. This cipher can assist in upgrading old ciphertext to the new format. See the Upgrading from 0.6.x guide for usage."},{"ref":"readme.html","title":"Cloak","type":"extras","doc":"Cloak Cloak is an Elixir encryption library that implements several best practices and conveniences for Elixir developers: Random IVs Tagged ciphertexts Elixir-native configuration"},{"ref":"readme.html#documentation","title":"Cloak - Documentation","type":"extras","doc":"Hex Documentation (Includes installation guide) How to upgrade from Cloak 0.9.x to 1.0.x"},{"ref":"readme.html#examples","title":"Cloak - Examples","type":"extras","doc":"Encrypt / Decrypt {:ok, ciphertext} = MyApp.Vault.encrypt(&quot;plaintext&quot;) # =&gt; {:ok, &lt;&lt;1, 10, 65, 69, 83, 46, 71, 67, 77, 46, 86, 49, 45, 1, 250, 221, # =&gt; 189, 64, 26, 214, 26, 147, 171, 101, 181, 158, 224, 117, 10, 254, 140, 207, # =&gt; 215, 98, 208, 208, 174, 162, 33, 197, 179, 56, 236, 71, 81, 67, 85, 229, # =&gt; ...&gt;&gt;} MyApp.Vault.decrypt(ciphertext) # =&gt; {:ok, &quot;plaintext&quot;} Reencrypt With New Algorithm/Key &quot;plaintext&quot; |&gt; MyApp.Vault.encrypt!(:aes_gcm) |&gt; MyApp.Vault.decrypt!() |&gt; MyApp.Vault.encrypt!(:aes_ctr) |&gt; MyApp.Vault.decrypt!() # =&gt; &quot;plaintext&quot; Configuration config :my_app, MyApp.Vault, ciphers: [ aes_gcm: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: &lt;&lt;...&gt;&gt;}, aes_ctr: {Cloak.Ciphers.AES.CTR, tag: &quot;AES.CTR.V1&quot;, key: &lt;&lt;...&gt;&gt;} ]"},{"ref":"readme.html#features","title":"Cloak - Features","type":"extras","doc":"Random Initialization Vectors (IV) Every strong encryption algorithm recommends unique initialization vectors. Cloak automatically generates unique vectors using :crypto.strong_rand_bytes, and includes the IV in the ciphertext. This greatly simplifies storage and is not a security risk. Tagged Ciphertext Each ciphertext contains metadata about the algorithm and key which was used to encrypt it. This allows Cloak to automatically select the correct key and algorithm to use for decryption for any given ciphertext. This makes key rotation much easier, because you can easily tell whether any given ciphertext is using the old key or the new key. Elixir-Native Configuration Cloak works through Vault modules which you define in your app, and add to your supervision tree. You can have as many vaults as you wish running simultaneously in your project. (This works well with umbrella apps, or any runtime environment where you have multiple OTP apps using Cloak) Ecto Support You can use Cloak to transparently encrypt Ecto fields, using cloak_ecto."},{"ref":"readme.html#security-notes","title":"Cloak - Security Notes","type":"extras","doc":"Cloak is built on Erlang&#39;s crypto library, and therefore inherits its security. You can implement your own cipher modules to use with Cloak, which may use any other encryption algorithms of your choice."},{"ref":"install.html","title":"Install Cloak","type":"extras","doc":"Installing Cloak This guide will walk you through installing Cloak in your project. Add the Dependency First, add :cloak to your dependencies in mix.exs: {:cloak, &quot;1.0.0&quot;} Run mix deps.get to fetch the dependency. Generate a Key You&#39;ll need a secret key for encryption. This is easy to generate in the IEx console. $ iex iex&gt; 32 |&gt; :crypto.strong_rand_bytes() |&gt; Base.encode64() &quot;aJ7HcM24BcyiwsAvRsa3EG3jcvaFWooyQJ+91OO7bRU=&quot; This will generate a relatively strong encryption 256-bit encryption key encoded with Base64. Create a Vault Next, create a Cloak.Vault for your project. defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app end Configure it as shown in the Cloak.Vault documentation, with at least one active cipher. Note that the :key needs to be decoded from Base64 encoding into its raw binary form. config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: Base.decode64!(&quot;your-key-here&quot;)} ] If you want to fetch keys from system vars, you should use the init/1 callback to configure the vault instead: # Assumes that you have a CLOAK_KEY environment variable containing a key in # Base64 encoding. # # export CLOAK_KEY=&quot;A7x+qcFD9yeRfl3GohiOFZM5bNCdHNu27B0Ozv8X4dE=&quot; defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl GenServer def init(config) do config = Keyword.put(config, :ciphers, [ default: {Cloak.Ciphers.AES.GCM, tag: &quot;AES.GCM.V1&quot;, key: decode_env!(&quot;CLOAK_KEY&quot;)} ]) {:ok, config} end defp decode_env!(var) do var |&gt; System.get_env() |&gt; Base.decode64!() end end Finally, add your vault to your supervision tree. children = [ MyApp.Vault ]"},{"ref":"install.html#usage","title":"Install Cloak - Usage","type":"extras","doc":"You can now encrypt and decrypt values using your Vault. {:ok, ciphertext} = MyApp.Vault.encrypt(&quot;plaintext&quot;) # =&gt; {:ok, &lt;&lt;1, 10, 65, 69, 83, 46, 71, 67, 77, 46, 86, 49, 93, 140, 255, 234, 1, 195, 125, 112, 121, 186, 169, 185, 129, 122, 237, 161, 160, 24, 166, 48, 224, 230, 53, 194, 251, 175, 215, 10, 186, 130, 61, 230, 176, 102, 213, 209, ...&gt;&gt;} MyApp.Vault.decrypt(ciphertext) {:ok, &quot;plaintext&quot;} By default, the first configured key will be used. You can use a specific key to use by referencing its label: MyApp.Vault.encrypt(&quot;plaintext&quot;, :default) Decryption will use the metadata embedded in the ciphertext to decide which configured key to use."},{"ref":"install.html#usage-with-ecto","title":"Install Cloak - Usage with Ecto","type":"extras","doc":"If you want to use Cloak to automatically encrypt and decrypt fields in your Ecto schemas, see cloak_ecto."},{"ref":"generate_keys.html","title":"Generate Encryption Keys","type":"extras","doc":"How to Generate Encryption Keys The easiest way to generate a key is via IEx. $ iex iex&gt; 32 |&gt; :crypto.strong_rand_bytes() |&gt; Base.encode64() &quot;HXCdm5z61eNgUpnXObJRv94k3JnKSrnfwppyb60nz6w=&quot; This will generate a strong 256-bit key encoded with Base64."},{"ref":"0-9-x_to_1-0-x.html","title":"0.9.x to 1.0.x","type":"extras","doc":"How To Upgrade From 0.9.x to 1.0.x Cloak 1.0.0 separated the Ecto components into a separate package: cloak_ecto. This required some breaking name changes. Cloak&#39;s default JSON library is now Jason, not Poison All Ecto-related modules now have the prefix Cloak.Ecto. Cloak&#39;s Ecto types have a new naming scheme: Cloak.Fields.Binary became Cloak.Ecto.Binary. mix cloak.migrate has been renamed to mix cloak.migrate.ecto. Install cloak_ecto 1.0.0 Replace your cloak dependency with cloak_ecto. {:cloak_ecto, &quot;1.0.0&quot;} Configure JSON Library If you have not specified the :json_library setting in your Vault, and want to continue using Poison, you will need to add that setting: config :my_app, MyApp.Vault, json_library: Poison, # ... Otherwise, you will need to add Jason to your dependencies: {:jason, &quot;~&gt; 1.0&quot;} Update Cloak Field References Update all your Cloak field references from the old naming scheme to the new. For example: use Cloak.Fields.Binary, vault: MyApp.Vault Should now become: use Cloak.Ecto.Binary, vault: MyApp.Vault Update Migration References If you use Cloak.Migrator directly, rename your references to Cloak.Ecto.Migrator instead. If you use mix cloak.migrate, rename any references to mix cloak.migrate.ecto instead.."},{"ref":"0-8-x_to_0-9-x.html","title":"0.8.x to 0.9.x","type":"extras","doc":"How To Upgrade From 0.8.x to 0.9.x Cloak 0.9.0 encrypts and decrypts data up to 200% faster than 0.8.0. This result is achieved by caching configuration data in an ETS table. Breaking Change: You must now add your Cloak.Vault to your supervision tree."},{"ref":"0-8-x_to_0-9-x.html#update-your-dependency","title":"0.8.x to 0.9.x - Update Your Dependency","type":"extras","doc":"Update your cloak dependency to 0.9.0 or higher: {:cloak, &quot;~&gt; 0.9.2&quot;} Supervise Your Vault Add your vault to your supervision tree: children = [ MyApp.Vault ] Update init/1 callback implementation If you are fetching the keys from system vars, you should change the implementation of the Cloak.Vault.init/1 callback to GenServer.init/1 defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl GenServer def init(config) do (...) end end"},{"ref":"0-7-x_to_0-8-x.html","title":"0.7.x to 0.8.x","type":"extras","doc":"How To Upgrade From 0.7.x to 0.8.x Cloak 0.8 introduces a new feature with no breaking changes. mix cloak.migrate now supports binary primary keys. It iterates through database tables using a cursor, instead of min/max ID. Cloak.CustomCursor allows you to customize fields used for the cursor."},{"ref":"0-7-x_to_0-8-x.html#update-your-dependency","title":"0.7.x to 0.8.x - Update Your Dependency","type":"extras","doc":"Update your cloak dependency to 0.8 or later. {:cloak, &quot;~&gt; 0.8.0&quot;} Most users will not need to do anything more. See the Cloak.CustomCursor docs for more details."},{"ref":"0-6-x_to_0-7-x.html","title":"0.6.x to 0.7.x","type":"extras","doc":"How To Upgrade From 0.6.x to 0.7.x Cloak 0.7 introduced a number of important changes. Encryption is now performed through Cloak.Vault modules Ciphertext no longer contains a module_tag Ecto types are now local to your project You no longer need an :encryption_version field"},{"ref":"0-6-x_to_0-7-x.html#install-cloak-0-7","title":"0.6.x to 0.7.x - Install Cloak 0.7","type":"extras","doc":"Update your cloak dependency to 0.7 or later. {:cloak, &quot;~&gt; 0.7.0&quot;}"},{"ref":"0-6-x_to_0-7-x.html#create-a-vault","title":"0.6.x to 0.7.x - Create a Vault","type":"extras","doc":"Create a vault module for your project. (Or more than one, if you want!) defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app end You will need to move your existing configuration to the vault. For example, if you had this configuration: config :cloak, Cloak.AES.CTR, tag: &quot;AES&quot;, default: true, keys: [ %{tag: &lt;&lt;1&gt;&gt;, key: :base64.decode(&quot;...&quot;), default: true} ] You would convert it to the following: config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.CTR, tag: &quot;AES.V2&quot;, key: Base.decode64(&quot;...&quot;)}, retired: {Cloak.Ciphers.Deprecated.AES.CTR, module_tag: &quot;AES&quot;, tag: &lt;&lt;1&gt;&gt;, key: Base.decode64(&quot;...&quot;)} ] Notice that the tag: &quot;AES&quot; became module_tag: &quot;AES&quot; in the :retired cipher configuration. Alternatively, if your keys are stored in environment variables, you could configure the vault using the Cloak.Vault.init/1 callback: defmodule MyApp.Vault do use Cloak.Vault, otp_app: :my_app @impl Cloak.Vault def init(config) do config = Keyword.put(config, :ciphers, [ default: {Cloak.Ciphers.AES.CTR, tag: &quot;AES.V2&quot;, key: decode_env(&quot;CLOAK_KEY&quot;)}, retired: {Cloak.Ciphers.Deprecated.AES.CTR, module_tag: &quot;AES&quot;, tag: &lt;&lt;1&gt;&gt;, key: decode_env(&quot;CLOAK_KEY&quot;)} ]) {:ok, config} end defp decode_env(var) do var |&gt; System.get_env() |&gt; Base.decode64!() end end"},{"ref":"0-6-x_to_0-7-x.html#create-project-specific-ecto-types","title":"0.6.x to 0.7.x - Create Project-Specific Ecto Types","type":"extras","doc":"For each type of encrypted field you have, define a local type. For example, if you had the following schema: defmodule MyApp.Accounts.User do use Ecto.Schema import Ecto.Changeset schema &quot;users&quot; do field :name, Cloak.EncryptedBinaryField, field :encryption_version end @doc false def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:name]) |&gt; put_change(:encryption_version, Cloak.version()) end end You would define a project-specific field: defmodule MyApp.Encrypted.Binary do use Cloak.Fields.Binary, vault: MyApp.Vault end And then replace Cloak.EncryptedBinaryField in your schema: schema &quot;users&quot; do field :name, MyApp.Encrypted.Binary, field :encryption_version end Finally, you&#39;d remove the :encryption_version field as it is no longer needed. # In migration... alter table(:users) do remove :encryption_version end # In your changeset... @doc false def changeset(struct, attrs \\\\ %{}) do struct |&gt; cast(attrs, [:name]) end"},{"ref":"0-6-x_to_0-7-x.html#migrate-existing-data","title":"0.6.x to 0.7.x - Migrate Existing Data","type":"extras","doc":"To convert ciphertext en masse from the old v0.6 format to the new v0.7 format, you&#39;ll need to run mix cloak.migrate as shown in its documentation. mix cloak.migrate -r MyApp.Repo -s MyApp.Schema"},{"ref":"0-6-x_to_0-7-x.html#remove-retired-cipher","title":"0.6.x to 0.7.x - Remove :retired Cipher","type":"extras","doc":"Now that the data has been migrated to the new v0.7 format, you can remove the :retired cipher from your configuration. config :my_app, MyApp.Vault, ciphers: [ default: {Cloak.Ciphers.AES.CTR, tag: &quot;AES.V2&quot;, key: Base.decode64(&quot;...&quot;)} ]"}]